package com.hzya.frame.plugin.a8bill.plugin;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapBuilder;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.ningboBankTreasury.dto.req.AccTransactionReq;
import com.hzya.frame.ningboBankTreasury.dto.res.AccTransactionRes;
import com.hzya.frame.ningboBankTreasury.util.NingboBankUtil;
import com.hzya.frame.plugin.a8bill.dao.IPayBillPluginInitializerDao;
import com.hzya.frame.plugin.a8bill.entity.PayBillEntity;
import com.hzya.frame.plugin.a8bill.service.INingboBankPluginService;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description 查询宁波银行账户交易明细 保存到OA
 * @Author xiangerlin
 * @Date 2024/7/25 16:24
 **/
public class AccLogPluginInitializer extends PluginBaseEntity {
    Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private INingboBankPluginService ningboBankPluginService;
    @Value("${OA.data_source_code}")
    private String oa_data_source_code;
    @Autowired
    private IPayBillPluginInitializerDao payBillDao;
    /***
     * 插件初始化方法
     * @Author 👻👻👻👻👻👻👻👻 gjh
     * @Date 2023-08-02 10:48
     * @Param []
     * @return void
     **/
    @Override
    public void initialize() {
        logger.info(getPluginLabel() + "執行初始化方法initialize()");
    }

    /****
     * 插件销毁方法
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2023-08-02 10:48
     * @return void
     **/
    @Override
    public void destroy() {
        logger.info(getPluginLabel() + "執行銷毀方法destroy()");
    }

    /****
     * 插件的ID
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2023-08-02 10:48
     * @return void
     **/
    @Override
    public String getPluginId() {
        return "AccLogPlugin";
    }

    /****
     * 插件的名称
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2023-08-02 10:48
     * @return void
     **/
    @Override
    public String getPluginName() {
        return "查询交易明细插件";
    }

    /****
     * 插件的显示值
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2023-08-02 10:48
     * @return void
     **/
    @Override
    public String getPluginLabel() {
        return "AccLogPlugin";
    }

    /***
     * 插件类型 1、场景插件
     * @Author 👻👻👻👻👻👻👻👻 gjh
     * @Date 2023-08-02 14:01
     * @Param []
     * @return java.lang.String
     **/
    @Override
    public String getPluginType() {
        return "1";
    }

    /***
     * 执行业务代码
     * @Author 👻👻👻👻👻👻👻👻 gjh
     * @Date 2023-08-07 11:20
     * @param requestJson  执行业务代码的参数
     * @return void
     **/
    @Override
    public JsonResultEntity executeBusiness(JSONObject requestJson) throws Exception {
        //查询银行账户列表
        PayBillEntity oaBillEntity = new PayBillEntity();
        oaBillEntity.setDataSourceCode(oa_data_source_code);
        List<HashMap<String, Object>> hashMaps = payBillDao.getOaBillData("SELECT field0002 from formmain_0053",oaBillEntity);
        List<String> accList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(hashMaps)){
            for (HashMap<String, Object> hashMap : hashMaps) {
                String field0002 = String.valueOf(hashMap.get("field0002"));
                accList.add(field0002);
            }
        }
        //测试银行卡号
//        accList = Convert.toList(String.class,"86041110000110293,86041110000109337,6223160030547665,6214180002001411023,6214180000002609736");
        //正式银行卡号
        accList = Convert.toList(String.class,"80010122000156204,80010122000263453,80012025000028568,80012029000027878,80015025000004736,359000100100541514,359001400100031916,359001100100008396,80010122000508721,80010122000170773,359060100100185025,80010122000927103,80010122000950933,80012025000144507,80015025000020731");
        //1、查询宁波银行的交易记录
        AccTransactionReq accTransactionReq = new AccTransactionReq();
        accTransactionReq.setCdSign(NingboBankUtil.SHOURU);
        accTransactionReq.setBeginDate(DateUtil.format(DateUtil.yesterday(),"yyyy-MM-dd"));
        accTransactionReq.setEndDate(DateUtil.today());
        accTransactionReq.setBankAccList(accList);
        Map<String, String> headerMap = MapBuilder.<String, String>create(true)
                .put("apiCode", "8000360002")
                .put("publicKey","ZJYAWb7lhAUTYqekPkU+uHJv1/ObJxb7dT7sD8HPRDGAgyhCe7eDIk+3zDUT+v578prj")
                .put("secretKey","fviZnLBsQUAGF8w8FSOdJi7XlIm/XAZclMxRagDLfTyJFlvnIBF3w66Hrpfzs8cYj3JzOP8MtA1LSGvL+2BWG8c/o7DKi92S4mr3zcGearA=")
                .put("appId","800036")
                .build();
        String body = HttpRequest.post("http://127.0.0.1:8901/kangarooDataCenterV3/entranceController/externalCallInterface").addHeaders(headerMap).body(JSONObject.toJSONString(accTransactionReq)).timeout(60000).execute().body();
        System.out.println(body);
        if (StrUtil.isNotEmpty(body)){
            JSONObject response = JSONObject.parseObject(body);
            if (response.getBoolean("flag")){
                List<AccTransactionRes> accTransactionResList = JSONObject.parseArray(response.getJSONObject("attribute").getString("attribute"), AccTransactionRes.class);
                //2、保存到OA底表
                if (CollectionUtils.isNotEmpty(accTransactionResList)){
                    ningboBankPluginService.saveAccLog(accTransactionResList);
                }
                return BaseResult.getSuccessMessageEntity(getPluginName()+"执行成功:{}",accTransactionResList);
            }
        }
        return BaseResult.getSuccessMessageEntity(getPluginName()+"执行成功");
    }
}
