package com.hzya.frame.plugin.a8bill.plugin;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.plugin.a8bill.dao.IPayBillPluginInitializerDao;
import com.hzya.frame.plugin.a8bill.entity.PayBillEntity;
import com.hzya.frame.plugin.a8bill.service.INingboBankPluginService;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.service.IIntegrationTaskLivingDetailsService;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;

/**
 * @Description OA付款申请审批完成后传u8c
 * @Author xiangerlin
 * @Date 2024/10/9 14:19
 **/
public class PayReqSyncU8CPluginInitializer extends PluginBaseEntity {
    Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private INingboBankPluginService ningboBankPluginService;
    @Resource
    private IIntegrationTaskLivingDetailsService taskLivingDetailsService;
    @Autowired
    private IPayBillPluginInitializerDao payBillDao;
    @Value("${OA.data_source_code}")
    private String oa_data_source_code;

    /***
     * 插件初始化方法
     * @Author 👻👻👻👻👻👻👻👻 gjh
     * @Date 2023-08-02 10:48
     * @Param []
     * @return void
     **/
    @Override
    public void initialize() {
        logger.info(getPluginLabel() + "執行初始化方法initialize()");
    }

    /****
     * 插件销毁方法
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2023-08-02 10:48
     * @return void
     **/
    @Override
    public void destroy() {
        logger.info(getPluginLabel() + "執行銷毀方法destroy()");
    }

    /****
     * 插件的ID
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2023-08-02 10:48
     * @return void
     **/
    @Override
    public String getPluginId() {
        return "PayReqSyncU8CPlugin";
    }

    /****
     * 插件的名称
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2023-08-02 10:48
     * @return void
     **/
    @Override
    public String getPluginName() {
        return "付款申请审批结果传U8C";
    }

    /****
     * 插件的显示值
     * @author 👻👻👻👻👻👻👻👻 gjh
     * @date 2023-08-02 10:48
     * @return void
     **/
    @Override
    public String getPluginLabel() {
        return "付款申请审批结果传U8C";
    }

    /***
     * 插件类型 1、场景插件
     * @Author 👻👻👻👻👻👻👻👻 gjh
     * @Date 2023-08-02 14:01
     * @Param []
     * @return java.lang.String
     **/
    @Override
    public String getPluginType() {
        return "1";
    }

    /***
     * 执行业务代码
     * @Author 👻👻👻👻👻👻👻👻 gjh
     * @Date 2023-08-07 11:20
     * @param requestJson  执行业务代码的参数
     * @return void
     **/
    @Override
    public JsonResultEntity executeBusiness(JSONObject requestJson) throws Exception {
        logger.info("======开始执行付款申请审批结果传U8C========");
        JSONObject requestJsonObj = JSON.parseObject(requestJson.toString(), JSONObject.class);
        String integration_task_living_details_id = requestJsonObj.getString("integration_task_living_details_id");
        IntegrationTaskLivingDetailsEntity oldMsg = new IntegrationTaskLivingDetailsEntity();
        if (StrUtil.isNotEmpty(integration_task_living_details_id)) {
            oldMsg = taskLivingDetailsService.get(integration_task_living_details_id);
        }
        String rootAppPk = oldMsg.getRootAppPk();
        try {
            StringBuffer sb = new StringBuffer();
            if (StrUtil.isNotEmpty(rootAppPk)) {
                sb.append(" and id = '" + rootAppPk + "'");
            }else {
                sb.append("and finishedflag = '1' and pushState is null");
            }
            PayBillEntity oaBillEntity = new PayBillEntity();
            oaBillEntity.setDataSourceCode(oa_data_source_code);
            StringBuffer stringBuffer = new StringBuffer();
            //付款单
            stringBuffer.append("SELECT * from v_hzya_payres_u8c WHERE 1= 1   ");
            stringBuffer.append(sb.toString());
            List<HashMap<String, Object>> hashMaps = payBillDao.getOaBillData(stringBuffer.toString(), oaBillEntity);
            for (HashMap<String, Object> hashMap : hashMaps) {
                JSONObject main = new JSONObject();
                for (String key : hashMap.keySet()) {
                    main.put(key, hashMap.get(key));
                }
                logger.info("组装数据前"+JSON.toJSONString(hashMaps));
                logger.info("组装数据前1"+main.toString());
                ningboBankPluginService.updateU8CStatus(main,integration_task_living_details_id);
            }
        } catch (Exception e) {
            logger.info("获取OA数据失败" + e.getMessage());
            e.printStackTrace();
        }
        return BaseResult.getSuccessMessageEntity(getPluginName()+"执行成功");
    }
}
