package com.hzya.frame.plugin.cbs8.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.map.MapBuilder;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.hzya.frame.cbs8.dto.req.*;
import com.hzya.frame.cbs8.dto.res.*;
import com.hzya.frame.cbs8.service.ICbs8Service;
import com.hzya.frame.cbs8.util.CBSUtil;
import com.hzya.frame.cbs8.util.CurrencyEnum;
import com.hzya.frame.cbs8.util.PayState;
import com.hzya.frame.plugin.cbs8.service.ICbsPluginService;
import com.hzya.frame.seeyon.cap4.form.dto.*;
import com.hzya.frame.seeyon.cbs8.entity.*;
import com.hzya.frame.seeyon.cbs8.service.*;
import com.hzya.frame.seeyon.entity.CtpAttachmentEntity;
import com.hzya.frame.seeyon.entity.CtpFileEntity;
import com.hzya.frame.seeyon.service.ICtpAttachmentService;
import com.hzya.frame.seeyon.util.RestUtil;
import com.hzya.frame.uuid.UUIDLong;
import com.hzya.frame.web.exception.BaseSystemException;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description
 * @Author xiangerlin
 * @Date 2024/6/17 08:46
 **/
public class CbsPluginServiceImpl implements ICbsPluginService {

    Logger logger = LoggerFactory.getLogger(CbsPluginServiceImpl.class);
    @Autowired
    private ICbs8Service cbs8Service;
    @Autowired
    private IPaymentService paymentService;
    @Autowired
    private ICbsLogService cbsLogService;
    @Autowired
    private ICtpAttachmentService ctpAttachmentService;
    @Autowired
    private ITransactionDetailService transactionDetailService;
    @Autowired
    private IAgentPaymentService agentPaymentService;
    @Autowired
    private IAgentPaymentDetailService agentPaymentDetailService;

    @Autowired
    private RestUtil restUtil;
    @Value("${cbs8.elec_path:}")
    private String elec_path;
    @Value("${OA.data_source_code:}")
    private String oa_data_source_code;
    /**
     * 支付申请
     *
     * @param paymentEntity
     */
    @Override
    public void applyPay(PaymentEntity paymentEntity) throws Exception{
        //查询待支付的列表
        if (null == paymentEntity){
            paymentEntity = new PaymentEntity();
        }
        paymentEntity.setDataSourceCode(oa_data_source_code);
        List<PaymentEntity> paymentList = paymentService.queryUnpaid(paymentEntity);
       /* List<PaymentEntity> paymentList = new ArrayList<>();
        paymentEntity.setReferenceNum("CL202406140001");
        paymentEntity.setPayAccount("655905707410000");
        paymentEntity.setPayBankName("");
        paymentEntity.setAmount("99");
        paymentEntity.setRevAccount("123456778");
        paymentEntity.setRevBankName("中国工商银行总行清算中心");
        paymentEntity.setRevBankType("ICB");
        paymentEntity.setRevAccountName("测试账户");
        paymentEntity.setCnapsCode("102100099996");
        paymentEntity.setPurpose("测试用途");
        paymentEntity.setBusType("202");
        paymentEntity.setCurrency("10");
        paymentEntity.setPurpose("测试用途");
        paymentList.add(paymentEntity);*/
        if (CollectionUtils.isNotEmpty(paymentList)){
            for (PaymentEntity pay : paymentList) {
                //调用支付申请接口
                PayResponseDTO payResponseDTO = cbs8Service.payApply(pay);
                boolean successed = payResponseDTO.getSuccessed();
                if (successed){
                    pay.setPayResult(PayState.p.getValue());
                }else {
                    pay.setPayResult("推送失败");
                }
                //4、更新OA表单
                pay.setDataSourceCode(oa_data_source_code);
                pay.setApplyCode(payResponseDTO.getBusNum());
                paymentService.updatePayState(pay);
                //5、记录操作日志
                savePayLog(pay,payResponseDTO);
            }
        }
    }

    /**
     * 保存支付申请日志
     *
     * @param entity
     * @throws Exception
     */
    @Override
    public void savePayLog(PaymentEntity entity,PayResponseDTO payResponseDTO) throws Exception {
        //4. 保存日志
        CbsLogEntity cbsLogEntity = new CbsLogEntity();
        cbsLogEntity.setTitle(entity.getTitle());
        cbsLogEntity.setPay_company(entity.getPayCompany());
        cbsLogEntity.setPayee(entity.getRevAccountName());
        cbsLogEntity.setAmount(entity.getAmount());
        cbsLogEntity.setOa_id(entity.getOaId());
        cbsLogEntity.setBill_code(Convert.toStr(entity.getReferenceNumNew(),entity.getReferenceNum()));
        cbsLogEntity.setTab_name_ch(entity.getBillName());
        cbsLogEntity.setTab_name_en(entity.getTableName());
        Boolean successed = payResponseDTO.getSuccessed();
        if (successed){
            cbsLogEntity.setPay_state(PayState.p.getValue());
            cbsLogEntity.setApply_state(PayState.two.getValue());
            cbsLogEntity.setCbs_apply_code(payResponseDTO.getBusNum());
            cbsLogEntity.setSuccessed("true");
            entity.setPayResult(PayState.p.getValue());
        }else {
            cbsLogEntity.setPay_state("推送失败");
            cbsLogEntity.setMessage(payResponseDTO.getErrorMsg());
            cbsLogEntity.setSuccessed("false");
            entity.setPayResult("推送失败");
        }
        cbsLogService.saveLog(cbsLogEntity);
    }

    /**
     * 查询支付申请的交易结果
     *
     * @param cbsLogEntity
     * @throws Exception
     */
    @Override
    public void queryResult(CbsLogEntity cbsLogEntity) throws Exception {
        if (null == cbsLogEntity){
            cbsLogEntity = new CbsLogEntity();
        }
        cbsLogEntity.setDataSourceCode(oa_data_source_code);
        // 1、查询支付中的日志
        List<CbsLogEntity> inPayList = cbsLogService.queryInPayment(cbsLogEntity);
        if (CollectionUtils.isNotEmpty(inPayList)){
            for (CbsLogEntity entity : inPayList) {
                try {
                    List<PayResultResDTO> payResultResList = cbs8Service.queryPayResult(new PayResultRequestDTO(entity.getBill_code()));
                    if (CollectionUtils.isNotEmpty(payResultResList)){
                        PayResultResDTO payResultResDTO = payResultResList.get(0);
                        //支付申请状态
                        String status = payResultResDTO.getStatus();
                        //支付状态
                        String pay_status = payResultResDTO.getPayStatus();
                        //不等于支付中的时候 更新支付状态
                        if (!PayState.p.getType().equals(pay_status)){
                            //如果支付状态为空，保存支付申请状态，如果支付状态不为空，则保存支付状态
                            PaymentEntity paymentEntity = new PaymentEntity();
                            paymentEntity.setOaId(entity.getOa_id());
                            paymentEntity.setApplyCode(entity.getCbs_apply_code());
                            paymentEntity.setDataSourceCode(oa_data_source_code);
                            List<PaymentEntity> paymentList = paymentService.query(paymentEntity);
                            if (CollectionUtils.isNotEmpty(paymentList)){
                                paymentEntity = paymentList.get(0);
                                if (StrUtil.isEmpty(pay_status)) {
                                    //支付申请状态 支付状态和支付申请状态用一个
                                    paymentEntity.setPayResult(PayState.payStateGetValue(status));
                                } else {
                                    //支付状态 支付状态和支付申请状态用一个
                                    paymentEntity.setPayResult(PayState.payStateGetValue(pay_status));
                                }
                                if (StrUtil.isNotEmpty(pay_status) && pay_status.equals(PayState.g.getType())) {
                                    //支付时间
                                    paymentEntity.setPayDate(CBSUtil.convertTimestampToString(payResultResDTO.getPayDate()));
                                }
                                //更新视图单据状态
                                paymentEntity.setDataSourceCode(oa_data_source_code);
                                paymentService.updatePayState(paymentEntity);
                                //更新日志表状态
                                entity.setPay_state(paymentEntity.getPayResult());
                                entity.setApply_state(PayState.payStateGetValue(status));
                                entity.setDataSourceCode(oa_data_source_code);
                                cbsLogService.updateLog(entity);
                            }
                        }
                    }
                }catch (Exception e){
                 e.printStackTrace();
                 logger.error("查询交易结果出错",e);
                }
            }
        }
    }

    /**
     * 电子回单查询 并上传OA
     *
     * @param requestJson
     * @throws Exception
     */
    @Override
    public void elecBillUpload(JSONObject requestJson) throws Exception {
        //查询支付成功 没有电子回单的数据
        PaymentEntity paymentEntity = new PaymentEntity();
       // List<PaymentEntity> paymentList = paymentService.queryElecIsNull(paymentEntity);
        paymentEntity.setPayDate("2024-06-20");
        paymentEntity.setReferenceNum("41");
        List<PaymentEntity> paymentList = Arrays.asList(paymentEntity);
        if (CollectionUtils.isNotEmpty(paymentList)) {
            for (PaymentEntity pay : paymentList) {
                try {
                    String payDate = DateUtil.format(DateUtil.parse(pay.getPayDate()), "yyyy-MM-dd");
                    //查询cbs电子回单
                    List<ElecResponseDTO> elecResList = cbs8Service.queryElecBill(new ElecRequestDTO(payDate,DateUtil.today(),pay.getReferenceNum()));
                    if (CollectionUtils.isNotEmpty(elecResList)){
                        ElecResponseDTO elecResponseDTO = elecResList.get(0);
                        String bucketFileUrl = elecResponseDTO.getBucketFileUrl();
                        String bucketFileName = elecResponseDTO.getBucketFileName();
                        //上传电子回单到OA
                        HttpUtil.downloadFile(bucketFileUrl, FileUtil.file(elec_path));//附件下载
                        String pdfUrl = elec_path + bucketFileName;
                        File file = new File(pdfUrl);
                        if (file.exists()) {
                            CtpFileEntity cpFileEntity =  new CtpFileEntity();
                            cpFileEntity.setFile(file);
                            cpFileEntity.setDataSourceCode(oa_data_source_code);
                            JSONObject jsonObjectUpload = restUtil.fileUpload(file,"8000240005");
                            String file_url = jsonObjectUpload.getString("fileUrl");
                            if (null != jsonObjectUpload && StrUtil.isNotEmpty(file_url)) {
                                String sub_reference = String.valueOf(UUIDLong.longUUID());
                                pay.setReceipt(sub_reference);
                                //更新表单的电子回单值
                                paymentService.updateElec(pay);
                                //保存附件关系
                                CtpAttachmentEntity ctpAttachmentEntity = new CtpAttachmentEntity();
                                ctpAttachmentEntity.setFile_url(file_url);
                                ctpAttachmentService.saveAttachment(file_url, pay.getSummaryId(), sub_reference);
                            }
                            //删除本地临时文件
                            file.delete();
                        }
                    }
                }catch (Exception e){
                    logger.error("电子回单查询出错",e);
                }
            }
        }
    }

    /**
     * 查询交易明细
     * transactionDetailReqDTO.currentPage 、 transactionDetailReqDTO.pageSize 必填
     * @param transactionDetailReqDTO
     */
    @Override
    public List<TransactionDetailDTO> queryTransactionDetail(TransactionDetailReqDTO transactionDetailReqDTO) {
        boolean hasNextPage = true;//是否有下一页
        int currentPage = transactionDetailReqDTO.getCurrentPage();
        int pageSize = transactionDetailReqDTO.getPageSize();
        if (currentPage == 0){
            currentPage = CBSUtil.DEFAULT_CURRENT_PAGE;//页码
        }
        if (pageSize == 0){
            pageSize = CBSUtil.DEFAULT_PAGE_SIZE;//每页条数
            transactionDetailReqDTO.setPageSize(pageSize);
        }
        List<TransactionDetailDTO> resultList = new ArrayList<>();
        do{
            transactionDetailReqDTO.setCurrentPage(currentPage);//页码
            currentPage++;//页码自增
            hasNextPage = false;//保护功能，防止出现死循环
            CbsResDataDTO dataDTO = cbs8Service.queryTransactionDetail(transactionDetailReqDTO);
            if (null != dataDTO){
                hasNextPage = dataDTO.getHasNextPage();
                List<TransactionDetailDTO> transactionDetailDTOList = CBSUtil.convertJsonArrayToList(dataDTO.getList(), TransactionDetailDTO.class);
                resultList.addAll(transactionDetailDTOList);
            }
        }while (hasNextPage);

        return resultList;
    }

    /**
     * 代发代扣 支付申请
     *
     * @param paymentEntity

     */
    @Override
    public void applyAgentPay(AgentPaymentEntity paymentEntity) {
        try {
            if (null != paymentEntity ){
                paymentEntity.setDataSourceCode(oa_data_source_code);
                List<AgentPaymentEntity> agentPaymentList = agentPaymentService.queryUnpaid(paymentEntity);
                if (CollectionUtils.isNotEmpty(agentPaymentList)){
                    for (AgentPaymentEntity agentPay : agentPaymentList) {
                        AgentPaymentDetailEntity detailEntity = new AgentPaymentDetailEntity();
                        detailEntity.setFormmainId(agentPay.getOaId());
                        detailEntity.setDataSourceCode(oa_data_source_code);
                        List<AgentPaymentDetailEntity> agentPaymentDetailList = agentPaymentService.queryDetails(detailEntity);
                        if (CollectionUtils.isNotEmpty(agentPaymentDetailList)){
                            PaymentApplySubmitReqDTO paymentApplySubmitReqDTO = BeanUtil.copyProperties(agentPay,PaymentApplySubmitReqDTO.class);
                            List<PaymentApplyAgentDTO> paymentApplyAgentList = new ArrayList<>();
                            for (AgentPaymentDetailEntity detail : agentPaymentDetailList) {
                                PaymentApplyAgentDTO detailDTO = BeanUtil.copyProperties(detail,PaymentApplyAgentDTO.class);
                                paymentApplyAgentList.add(detailDTO);
                            }
                            //招行这里要传203
                            paymentApplySubmitReqDTO.setBankExtend5("203");
                            PayResponseDTO payResponseDTO =  cbs8Service.agentPayApply(paymentApplySubmitReqDTO,paymentApplyAgentList);
                            if (null != payResponseDTO){
                                Boolean successed = payResponseDTO.getSuccessed();
                                AgentPaymentEntity pay = new AgentPaymentEntity();
                                pay.setOaId(paymentEntity.getOaId());
                                pay.setDataSourceCode(oa_data_source_code);
                                if (successed){
                                    //更新申请单号到OA
                                    pay.setApplyCode(payResponseDTO.getBusNum());
                                    pay.setPayResult("审批中");
                                }else {
                                    pay.setPayResult(payResponseDTO.getErrorMsg());
                                }
                                agentPaymentService.updateResult(pay);
                                System.out.println(JSONObject.toJSONString(payResponseDTO));
                            }
                        }
                    }
                }
            }else {
                throw new BaseSystemException("参数不能为空");
            }
        }catch (Exception e){
            logger.error("代发代扣出错:{}",e);
        }
    }

    /**
     * 代发代扣 结果详情查询
     *
     * @param agentPayResultRequestDTO
     * @return
     */
    @Override
    public AgentPayResultResDTO agentPayResult(AgentPayResultRequestDTO agentPayResultRequestDTO) {
        try {
            if (null != agentPayResultRequestDTO && StrUtil.isNotEmpty(agentPayResultRequestDTO.getBusNum())){
                AgentPayResultResDTO agentPayResultResDTO = cbs8Service.agentPayResult(agentPayResultRequestDTO);
                //更新OA表单
                String busNum = agentPayResultResDTO.getBusNum();
                if (StringUtils.isNotEmpty(busNum)){
                    AgentPaymentEntity agentPaymentEntity = new AgentPaymentEntity();
                    agentPaymentEntity.setApplyCode(busNum);
                    agentPaymentEntity.setDataSourceCode(oa_data_source_code);
                    AgentPaymentEntity entity =  agentPaymentService.queryByApplyCode(agentPaymentEntity);
                    String pay_status = agentPayResultResDTO.getPayStatus();
                    String status = agentPayResultResDTO.getStatus();
                    if (null != entity){
                        AgentPaymentEntity result = new AgentPaymentEntity();
                        result.setOaId(entity.getOaId());
                        result.setDataSourceCode(oa_data_source_code);
                        //更新主表
                        if (StrUtil.isEmpty(pay_status)) {
                            result.setPayResult(PayState.payStateGetValue(status));//支付申请状态 支付状态和支付申请状态用一个
                        } else {
                            result.setPayResult(PayState.payStateGetValue(pay_status));//支付状态 支付状态和支付申请状态用一个
                        }
                        agentPaymentService.updateResult(result);
                        //更新明细表
                        List<AgentPayQueryDTO> agentDetails = agentPayResultResDTO.getAgentDetails();
                        for (AgentPayQueryDTO d : agentDetails) {
                            AgentPaymentDetailEntity detail = new AgentPaymentDetailEntity();
                            detail.setFormmainId(entity.getOaId());
                            detail.setDtlAmount(d.getDtlAmount());
                            detail.setDtlCnapsCode(d.getDtlCnapsCode());
                            detail.setDtlRevName(d.getDtlRevName());
                            detail.setDtlSeqNum(Integer.valueOf(d.getDtlSeqNum()));
                            detail.setPayResult(PayState.payStateGetValue(d.getDtlStatus()));
                            detail.setPayDate(CBSUtil.convertTimestampToString(d.getDtlPayTime()));
                            detail.setDataSourceCode(oa_data_source_code);
                            agentPaymentDetailService.updatePayResult(detail);
                        }
                    }
                }
                return agentPayResultResDTO;
            }
        }catch (Exception e){
            logger.error("代发代扣详情结果查询出错{}",e);
        }
        return null;
    }

    /**
     * 保存交易明细到OA底表
     *
     * @param transactionlList
     */
    @Override
    public void saveTransactionDetail(List<TransactionDetailDTO> transactionlList) {
        if (CollectionUtils.isNotEmpty(transactionlList)){
            //过滤已经保存过的数据
            //收款档案表的数据
            TransactionDetailEntity transactionDetailEntity = new TransactionDetailEntity();
            transactionDetailEntity.setBankTransactionDate(DateUtil.lastWeek().toDateStr());
            transactionDetailEntity.setDataSourceCode(oa_data_source_code);
            List<TransactionDetailEntity> transactionDetailList = transactionDetailService.querySerialNumber(transactionDetailEntity);
            //过滤已经保存的数据
            if (CollectionUtils.isNotEmpty(transactionDetailList)){
                //过滤transactionlList 去除已经存在transactionDetailList中的数据，条件为transactionSerialNumber相等
                if (CollectionUtils.isNotEmpty(transactionlList)){
                    transactionlList = transactionlList.stream()
                            .filter(item -> transactionDetailList.stream()
                                    .noneMatch(detailItem -> item.getTransactionSerialNumber().equals(detailItem.getTransactionSerialNumber())))
                            .collect(Collectors.toList());
                }
            }
            if (CollectionUtils.isNotEmpty(transactionlList)){
                //保存到OA底表
                for (TransactionDetailDTO dto : transactionlList) {
                    TransactionDetailEntity transactionDetail = new TransactionDetailEntity();
                    BeanUtil.copyProperties(dto,transactionDetail);
                    transactionDetail.setCurrency(CurrencyEnum.getChineseNameByCode(dto.getCurrency()));
                    if (NumberUtil.isNumber(dto.getBankTransactionDate())){//如果是时间戳 转换成日期字符串
                        transactionDetail.setBankTransactionDate(CBSUtil.convertTimestampToString(dto.getBankTransactionDate()));
                    }
                    transactionDetailService.restSave(transactionDetail);
                }
            }
        }
    }

    //这个方法没用，用的是8.0批量保存方式，我没有测试
    private void saveTransactionDetailTemp(List<TransactionDetailDTO> transactionDetailList) {
        if (CollectionUtils.isNotEmpty(transactionDetailList)){
            //过滤已经保存过的数据
            for (TransactionDetailDTO dto : transactionDetailList) {
                List<FormDataDTO> dataList = new ArrayList<>();
                FormDTO formDTO = new FormDTO();
                formDTO.setFormCode("formmain_0233");
                formDTO.setLoginName("yonyou");
                formDTO.setRightId("6603635988997229999.-8611088937958683581");
                String field0001=dto.getAccountNo();//我方银行账号
                String field0002=dto.getAccountName();//我方户名
                String field0003=dto.getOpenBank();//我方开户行
                String field0004=dto.getBankType();//我方银行类型
                String field0005=dto.getTransactionSerialNumber();//交易流水号
                String field0006=dto.getBankTransactionDate();//交易日期
                String field0007=dto.getBankSerialNumber();//银行流水号
                String field0008=dto.getCurrency();//币种
                String field0009=dto.getIncurredAmount();//收款金额
                String field0010=dto.getPurpose();//用途
                String field0011=dto.getDigest();//摘要
                String field0012=dto.getOppositeAccount();//对方账号
                String field0013=dto.getOppositeName();//对方户名
                String field0014=dto.getOppositeOpeningBank();//对方开户行
                String field0015=dto.getRemark();//备注
                //fields
                List<RecordFieldDTO> fields = new ArrayList<>();
                fields.add(new RecordFieldDTO("field0001",field0001,field0001));
                fields.add(new RecordFieldDTO("field0002",field0002,field0002));
                fields.add(new RecordFieldDTO("field0003",field0003,field0003));
                fields.add(new RecordFieldDTO("field0004",field0004,field0004));
                fields.add(new RecordFieldDTO("field0005",field0005,field0005));
                fields.add(new RecordFieldDTO("field0006",field0006,field0006));
                fields.add(new RecordFieldDTO("field0007",field0007,field0007));
                fields.add(new RecordFieldDTO("field0008",field0008,field0008));
                fields.add(new RecordFieldDTO("field0009",field0009,field0009));
                fields.add(new RecordFieldDTO("field0010",field0010,field0010));
                fields.add(new RecordFieldDTO("field0011",field0011,field0011));
                fields.add(new RecordFieldDTO("field0012",field0012,field0012));
                fields.add(new RecordFieldDTO("field0013",field0013,field0013));
                fields.add(new RecordFieldDTO("field0014",field0014,field0014));
                fields.add(new RecordFieldDTO("field0015",field0015,field0015));
                //masterTable
                MasterTableDTO masterTableDTO = new MasterTableDTO();
                masterTableDTO.setName("formmain_1284");
                RecordDTO recordDTO = new RecordDTO();
                recordDTO.setId(UUIDLong.longUUID());
                recordDTO.setFields(fields);
                masterTableDTO.setRecord(recordDTO);
                //dataList
                FormDataDTO formDataDTO = new FormDataDTO();
                formDataDTO.setMasterTable(masterTableDTO);
                dataList.add(formDataDTO);
                formDTO.setDataList(dataList);
            }
        }
    }

}
