package com.hzya.frame.ningboBankTreasury.enums;

/**
 * @Description 转账状态
 * @Author xiangerlin
 * @Date 2024/7/29 17:11
 **/
public enum PayStatusEnum {
    SUCCESS("0","交易成功"),
    // 失败状态
    FAILED_DELETED("-2", "已删除"),
    APPROVAL_REJECTED_CLOSED("-3", "审批打回交易关闭"),
    TRANSACTION_FAILED("2", "交易失败"),
    FUND_ALLOCATION_FAILED("4", "交易失败，资金下拨失败"),
    LINKAGE_PAYMENT_FAILED_REFUNDED("5", "联动支付失败，资金退回成功"),
    LINKAGE_PAYMENT_FAILED_REFUND_PENDING("6", "联动支付失败，资金退回查证中"),
    LINKAGE_PAYMENT_FAILED_REFUND_FAILED("96", "联动支付失败，资金退回失败"),
    PAYMENT_REQUEST_FAILED("112", "请款失败"),

    // 其他状态
    REJECTED("-1", "驳回"),
    PENDING_VERIFICATION("1", "交易待查证"),
    PENDING_SEND("3", "待发送"),
    PENDING_APPROVAL("11", "待审批"),
    APPROVAL_THROUGH("95", "审批通过"),
    TEMPORARY_STORAGE("7", "暂存"),
    WITHDRAWN_BY_OPERATOR("-4", "经办撤回"),
    PAYMENT_REQUEST_PENDING("111", "请款查证中"),
    PAYMENT_REQUEST_SUCCEEDED_PENDING_SEND("113", "请款成功，交易待发送"),
    LINKAGE_PAYMENT_SUCCEEDED_PENDING_SEND("114", "联动支付成功，智能支付交易待发送");

    // 枚举属性
    private final String code;
    private final String description;
    // 枚举构造函数
    PayStatusEnum(String code, String description) {
        this.code = code;
        this.description = description;
    }

    // 获取状态码


    public String getCode() {
        return code;
    }

    // 获取状态描述
    public String getDescription() {
        return description;
    }

    // 根据状态码查找枚举
    public static PayStatusEnum fromCode(String code) {
        for (PayStatusEnum status : PayStatusEnum.values()) {
            if (status.getCode().equals(code)) {
                return status;
            }
        }
        throw new IllegalArgumentException("Unknown code: " + code);
    }

    public static String getByCode(String code){
        for (PayStatusEnum status : PayStatusEnum.values()) {
            if (status.getCode().equals(code)) {
                return status.getDescription();
            }
        }
        throw new IllegalArgumentException("Unknown code: " + code);
    }

    /**
     * 是否为最终态
     * @param code
     * @return
     */
    public static boolean isFinally(String code){
        return SUCCESS.code.equals(code)
                || FAILED_DELETED.code.equals(code)
                || APPROVAL_REJECTED_CLOSED.code.equals(code)
                || TRANSACTION_FAILED.code.equals(code)
                || FUND_ALLOCATION_FAILED.code.equals(code)
                || LINKAGE_PAYMENT_FAILED_REFUNDED.code.equals(code)
                || LINKAGE_PAYMENT_FAILED_REFUND_PENDING.code.equals(code)
                || LINKAGE_PAYMENT_FAILED_REFUND_FAILED.code.equals(code)
                || PAYMENT_REQUEST_FAILED.code.equals(code);
    }
}
